// Copyright {YYYY} Brian Newman. All rights reserved.

package handlers

import (
	"database/sql"
	"fmt"
	"html/template"
	"net/http"
	"net/url"
	"path/filepath"

	"cdmnky.io/dev/templates/web/v1/local.pkg/src/sitevars"

	"go.cdmnky.io/v2/net/assets"
	"go.cdmnky.io/v2/net/router"
	"go.cdmnky.io/v2/net/session"
	UI "go.cdmnky.io/v2/ui"
)

// NewServer ...
func NewServer(ui *UI.UI, db *sql.DB, assets *assets.Assets, session *session.Manager, sitevars *sitevars.Sitevars) *Server {
	return &Server{
		ui:       ui,
		db:       db,
		assets:   assets,
		session:  session,
		sitevars: sitevars,
	}
}

// Server ...
type Server struct {
	ui       *UI.UI
	db       *sql.DB
	assets   *assets.Assets
	session  *session.Manager
	sitevars *sitevars.Sitevars
}

// Start ...
func (x *Server) Start() {
	r := router.New()

	x.assets.RegisterHandler(r)

	r.Handle("GET", "/", Public(x.HandleIndex, x.db, x.session))
	r.Handle("GET", "/style/style.css", x.HandleStyle)
	r.Handle("GET", "/scripts/script.js", x.HandleScript)

	http.ListenAndServe(fmt.Sprintf(":%v", x.sitevars.Port), r)
}

// HandleIndex ...
func (x *Server) HandleIndex(w http.ResponseWriter, r *http.Request, params url.Values) {

	// pagevars ...
	pagevars := struct {
		Sitevars   *sitevars.Sitevars
		Authorized string
	}{
		Sitevars:   x.sitevars,
		Authorized: IsAuthorized(w, r, x.session),
	}

	htmfile, err := x.assets.Get(filepath.Join("webassets", "html", "index.html"))
	if IsMobile(r.Header.Get("User-Agent")) {
		x.ui.Infoln("Mobile detected...")
		htmfile, err = x.assets.Get(filepath.Join("webassets", "html", "index-mobile.html"))
	}
	if err != nil {
		w.WriteHeader(403)
		return
	}

	t, err := template.New("posts").Parse(string(htmfile))
	if err != nil {
		w.WriteHeader(403)
		return
	}
	t.Execute(w, &pagevars)
}

// HandleStyle ...
func (x *Server) HandleStyle(w http.ResponseWriter, r *http.Request, params url.Values) {

	fpath := "webassets/style/desktop.css"
	if IsMobile(r.Header.Get("User-Agent")) {
		x.ui.Infoln("Mobile detected...")
		fpath = "webassets/style/mobile.css"
	}
	cssStyle, _ := x.assets.Get(fpath)
	cssTheme, _ := x.assets.Get("webassets/style/theme.css")

	css := string(cssStyle)
	css += string(cssTheme)

	w.Header().Set("Content-Type", "text/css")
	fmt.Fprint(w, css)
}

// HandleScript ...
func (x *Server) HandleScript(w http.ResponseWriter, r *http.Request, params url.Values) {

	jquery, _ := x.assets.Get("webassets/vendor/jquery/js/jquery.min.js")
	toolbox, _ := x.assets.Get("webassets/scripts/toolbox.js")
	jsadmin, _ := x.assets.Get("webassets/scripts/admin.js")
	jspublic, _ := x.assets.Get("webassets/scripts/public.js")

	js := string(jquery) + "\n"
	js += string(toolbox) + "\n"
	js += string(jsadmin) + "\n"
	js += string(jspublic) + "\n"

	w.Header().Set("Content-Type", "application/javascript")
	fmt.Fprint(w, js)
}
